==> tecfy_corev2_package-1.1.74.tar.gz: lib/views/components/entity/dropdown_advance.property.dart <==
part of tecfy_corev2_client_package;

class DropdownAdvancedEntity extends StatefulWidget {
  /// entity data if the [entityId] is null
  final Entity entity;

  /// entity dbName if the [entity] property is null
  final String entityId;

  /// url to fill the data
  final String? url;

  /// label string
  final String? label;

  /// label description
  final String? description;

  /// is required
  final bool isRequired;

  /// is propety is read only
  final bool readOnly;

  /// full model data
  final Map<String, dynamic> data;

  /// property
  final PropertyDropdown property;

  /// on changed
  final void Function(Map<String, dynamic>?)? onChanged;

  ///on open
  final void Function()? onOpen;

  /// on close
  final void Function()? onClose;

  DropdownAdvancedEntity(
    this.entityId, {
    required this.property,
    this.label,
    this.description,
    this.isRequired = false,
    this.readOnly = false,
    required this.entity,
    this.url,
    this.onChanged,
    required this.data,
    this.onOpen,
    this.onClose,
  });
  @override
  _DropdownAdvancedEntityState createState() => _DropdownAdvancedEntityState();
}

class _DropdownAdvancedEntityState extends TecfyState<DropdownAdvancedEntity> {
  Map<String, dynamic>? _initValue;
  Entity? _entity;
  // String? _entityPropDisplayName;

  @override
  void initState() {
    super.initState();
    _init();
  }

  _init() {
    try {
      if (widget.property.dbName.contains('.')) {
        if (MapService.getValue(widget.data, widget.property.dbName) != null) {
          _initValue = MapService.getValue(widget.data, widget.property.dbName);
        }
      } else {
        if (widget.data[widget.property.dbName] is Map<String, dynamic>? ||
            widget.data[widget.property.dbName] is Map<String, dynamic>)
          _initValue = widget.data[widget.property.dbName];
      }
      if (_entity == null) _entity = widget.entity;
      if (_initValue == null) {
        if (widget.data[widget.property.dbName] != null &&
            widget.data[widget.property.dbName]['_id'] == _initValue?['id']) {
          if (widget.data[widget.property.dbName] is Map<String, dynamic>? ||
              widget.data[widget.property.dbName] is Map<String, dynamic>)
            _initValue = widget.data[widget.property.dbName];
        }
      }
    } catch (err) {}
  }

  void _onItemSelectedFromList(BuildContext? context, Map<String, dynamic>? v) {
    try {
      var valueBefore = _initValue;
      if (v != null) {
        if (widget.url != null && (widget.url?.isNotEmpty ?? false)) {
          _initValue = v;
        } else {
          if (widget.property.includedPropertiesAll) {
            _initValue = v;
          } else {
            _initValue = {};
            _initValue!['_id'] = v['_id'];
            _initValue![_entity!.displayField.dbName] =
                v[_entity!.displayField.dbName];
            for (var propertyName in widget.property.includedProperties) {
              var val = MapService.getValue(v, propertyName);
              MapService.updateValue(_initValue!, propertyName, val);
            }
          }
        }
        // _initValue!['name'] = v['name'];
        // _initValue!['title'] = v['title'];
      } else {
        _initValue = v;
      }
      // widget.data[widget.property.dbName] = _initValue;
      if (widget.onChanged != null &&
          (_initValue != null || valueBefore != null)) {
        widget.onChanged?.call(_initValue);
      }
      MapService.updateValue(widget.data, widget.property.dbName, _initValue);
      setState(() {});
      context != null ? Navigator.pop(context) : print("null Navigator");
    } catch (err, trac) {
      print('xxxx error select from list $err');
      print(trac);
    }
  }

  _onSaveNewItem(BuildContext context, newDataRaw) async {
    try {
      var validateResult = EntityService.validateEntity(_entity, newDataRaw);
      if (!validateResult.success) {
        AppNavigator.showMessage(
            context, validateResult.message ?? 'Error', MessageType.error);
        return;
      }
      var addResult =
          await EntityService.saveEntity(_entity, newDataRaw, context, true);
      if (addResult.success) {
        _initValue = addResult.data;
        widget.data[widget.property.dbName] = _initValue;
        widget.onChanged?.call(_initValue!);
        Navigator.pop(context, true); // close the add new item's form
      } else {
        AppNavigator.showMessage(
            context, addResult.message ?? 'Error', MessageType.error);
      }
    } catch (err, trace) {
      print('XXXXXXXXX erorr adding item');
      print(err);
      print(trace);
      AppNavigator.showMessage(context, 'Error Adding Item', MessageType.error);
    }
  }

  _openSearchDialog() async {
    widget.onOpen?.call();
    await AppNavigator.showAppDialog(
        context,
        (listDialogContext) => Column(
              mainAxisAlignment: MainAxisAlignment.center,
              crossAxisAlignment: CrossAxisAlignment.center,
              children: <Widget>[
                Expanded(child: Container()),
                Container(
                    width: min(500, MediaQuery.of(context).size.width),
                    height: min(
                        800,
                        MediaQuery.of(context).size.height -
                            MediaQuery.of(context).padding.top -
                            MediaQuery.of(context).padding.bottom -
                            50),
                    child: Card(
                        child: Column(
                            mainAxisAlignment: MainAxisAlignment.start,
                            crossAxisAlignment: CrossAxisAlignment.stretch,
                            children: <Widget>[
                          Container(
                              color: ThemeService.backgroundSecondColor,
                              child: Row(
                                  mainAxisAlignment:
                                      MainAxisAlignment.spaceBetween,
                                  children: <Widget>[
                                    Padding(
                                      padding: EdgeInsets.only(left: 10),
                                      child: getLableWidget(),
                                    ),
                                    IconButton(
                                        icon: Icon(Icons.close),
                                        onPressed: () {
                                          widget.onClose?.call();
                                          Navigator.pop(listDialogContext);
                                        })
                                  ])),
                          Expanded(
                              child: EntityListSearchComponent(
                            property: widget.property,
                            entity: widget.entity,
                            url: widget.url ?? '',
                            data: widget.data,
                            // isFilter: widget.isFilter ,
                            // filterValue: widget.filterValue,
                            // filterField: widget.filterField,
                            urlFilterData: widget.data,
                            onClick: (v) =>
                                _onItemSelectedFromList(listDialogContext, v),
                          )),
                          TextButton(
                            child: TecfyText(trans('Add New Item'),
                                style: ThemeService.flatButtonTextStyle
                                    .copyWith(
                                        decoration: TextDecoration.underline)),
                            onPressed: () async {
                              Map<String, dynamic> newDataRaw =
                                  new Map<String, dynamic>();
                              var addNewResult =
                                  await AppNavigator.showAppDialogEx(
                                      listDialogContext,
                                      () => _getAddNewDialog(
                                          listDialogContext, newDataRaw),
                                      size: DialogBoxSizes.large);
                              if (addNewResult ?? false)
                                Navigator.pop(
                                    listDialogContext); // close the search form
                            },
                          )
                        ]))),
                Expanded(child: Container()),
              ],
            ));
    //loadData();
  }

  Widget _getAddNewDialog(addDialogContext, newDataRaw) {
    return Container(
      constraints: BoxConstraints(maxHeight: 500, maxWidth: 500),
      child: Center(
        child: Card(
          child: ListView(
            children: <Widget>[
              InLineForm(widget.entityId, true,
                  entity: _entity!,
                  inPopup: true,
                  data: newDataRaw,
                  onChanged: (v) {}),
              Padding(
                  padding: EdgeInsets.symmetric(horizontal: 20),
                  child: Row(
                    children: <Widget>[
                      Expanded(child: Container()),
                      Expanded(
                          child: AppButtonPost(
                              label: trans('Save'),
                              onPressed: () => _onSaveNewItem(
                                  addDialogContext, newDataRaw))),
                      SizedBox(width: 10),
                      Expanded(
                          child: Container(
                              height: 50.0,
                              child: TextButton(
                                  child: TecfyText(trans('Cancel'),
                                      style: ThemeService.flatButtonTextStyle),
                                  onPressed: () =>
                                      Navigator.pop(addDialogContext, false)))),
                      Expanded(child: Container()),
                    ],
                  )),
            ],
          ),
        ),
      ),
    );
  }

  @override
  Widget build(BuildContext context) {
    if (_initValue != widget.data[widget.property.dbName] || _initValue == null)
      _init();
    // print('----------- ${_entity!.displayField.dbName}');
    // if (_dataValue != null && _entity?.displayField.dbName != null)
    //   print("${_dataValue![_entity!.displayField.dbName]}");
    return _initValue != null
        ? Stack(children: [
            MaterialButton(
                padding: EdgeInsets.all(0),
                onPressed: widget.readOnly ? null : _openSearchDialog,
                child: widget.label == null
                    ? TecfyText(
                        _initValue == null
                            ? trans('Click To Select')
                            : (_initValue![_entity!.displayField.dbName] is Map
                                ? _initValue![_entity!.displayField.dbName][
                                        LocalConfigService.languagePrefix
                                            .toLowerCase()] ??
                                    'N/A'
                                : _initValue![_entity!.displayField.dbName]),
                        style: TextStyle(color: Colors.black, fontSize: 15))
                    : Column(
                        crossAxisAlignment: CrossAxisAlignment.start,
                        children: <Widget>[
                          SizedBox(height: 10),
                          getLableWidget(),
                          TecfyText(
                              _initValue == null
                                  ? trans('Click To Select')
                                  : (_initValue![_entity!.displayField.dbName]
                                          is Map
                                      ? _initValue![
                                                  _entity!.displayField.dbName][
                                              LocalConfigService.languagePrefix
                                                  .toLowerCase()]
                                          .toString()
                                      : _initValue![
                                              _entity!.displayField.dbName] ??
                                          'No Name'),
                              style:
                                  TextStyle(color: Colors.black, fontSize: 15)),
                          SizedBox(height: 10),
                          Container(
                              height: 1, color: ThemeService.textSubColor),
                          SizedBox(height: 5),
                        ],
                      )),
            widget.readOnly || widget.isRequired
                ? SizedBox()
                : Align(
                    alignment: Alignment.topLeft,
                    child: IconButton(
                        onPressed: () {
                          _onItemSelectedFromList(null, null);
                        },
                        icon: Icon(Icons.close, color: Colors.grey),
                        tooltip: trans("Clean")),
                  )
          ])
        : MaterialButton(
            padding: EdgeInsets.all(0),
            onPressed: _openSearchDialog,
            child: widget.label == null
                ? TecfyText(
                    _initValue == null
                        ? trans('Click To Select')
                        : (_initValue![_entity!.displayField.dbName] ?? 'N/A'),
                    style: TextStyle(color: Colors.black, fontSize: 15))
                : Column(
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children: <Widget>[
                      SizedBox(height: 10),
                      getLableWidget(),
                      TecfyText(
                          _initValue == null
                              ? trans('Click To Select')
                              : (_initValue![_entity!.displayField.dbName]
                                      ?.toString() ??
                                  'N/A'),
                          style: TextStyle(color: Colors.black, fontSize: 15)),
                      SizedBox(height: 10),
                      Container(height: 1, color: ThemeService.textSubColor),
                      SizedBox(height: 5),
                    ],
                  ));
  }

  Widget getLableWidget() {
    return Row(
      children: [
        Tooltip(
            message: widget.description ?? '',
            child: TecfyText(widget.label ?? '',
                style:
                    TextStyle(color: ThemeService.textSubColor, fontSize: 11))),
        widget.isRequired
            ? TecfyText('*', style: TextStyle(color: Colors.red, fontSize: 11))
            : SizedBox(),
      ],
    );
  }
}

==> tecfy_corev2_package-1.1.78.tar.gz: lib/views/components/entity/dropdown_advance.property.dart <==
part of tecfy_corev2_client_package;

class DropdownAdvancedEntity extends StatefulWidget {
  /// entity data if the [entityId] is null
  final Entity entity;

  /// entity dbName if the [entity] property is null
  final String entityId;

  /// url to fill the data
  final String? url;

  /// label string
  final String? label;

  /// label description
  final String? description;

  /// is required
  final bool isRequired;

  /// is propety is read only
  final bool readOnly;

  /// is propety is add or new
  final bool isAddNew;

  /// full model data
  final Map<String, dynamic> data;

  /// property
  final PropertyDropdown property;

  /// on changed
  final void Function(Map<String, dynamic>?)? onChanged;

  ///on open
  final void Function()? onOpen;

  /// on close
  final void Function()? onClose;

  DropdownAdvancedEntity(
    this.entityId, {
    required this.property,
    this.label,
    this.description,
    this.isRequired = false,
    this.readOnly = false,
    this.isAddNew = true,
    required this.entity,
    this.url,
    this.onChanged,
    required this.data,
    this.onOpen,
    this.onClose,
  });
  @override
  _DropdownAdvancedEntityState createState() => _DropdownAdvancedEntityState();
}

class _DropdownAdvancedEntityState extends TecfyState<DropdownAdvancedEntity> {
  Map<String, dynamic>? _initValue;
  Entity? _entity;
  // String? _entityPropDisplayName;

  @override
  void initState() {
    super.initState();
    _init();
  }

  _init() {
    try {
      if (widget.property.dbName.contains('.')) {
        if (MapService.getValue(widget.data, widget.property.dbName) != null) {
          _initValue = MapService.getValue(widget.data, widget.property.dbName);
        }
      } else {
        if (widget.data[widget.property.dbName] is Map<String, dynamic>? ||
            widget.data[widget.property.dbName] is Map<String, dynamic>)
          _initValue = widget.data[widget.property.dbName];
      }
      if (_entity == null) _entity = widget.entity;
      if (_initValue == null) {
        if (widget.data[widget.property.dbName] != null &&
            widget.data[widget.property.dbName]['_id'] == _initValue?['id']) {
          if (widget.data[widget.property.dbName] is Map<String, dynamic>? ||
              widget.data[widget.property.dbName] is Map<String, dynamic>)
            _initValue = widget.data[widget.property.dbName];
        }
      }
    } catch (err) {}
  }

  void _onItemSelectedFromList(BuildContext? context, Map<String, dynamic>? v) {
    try {
      var valueBefore = _initValue;
      if (v != null) {
        if (widget.url != null && (widget.url?.isNotEmpty ?? false)) {
          _initValue = v;
        } else {
          if (widget.property.includedPropertiesAll) {
            _initValue = v;
          } else {
            _initValue = {};
            _initValue!['_id'] = v['_id'];
            _initValue![_entity!.displayField.dbName] =
                v[_entity!.displayField.dbName];
            for (var propertyName in widget.property.includedProperties) {
              var val = MapService.getValue(v, propertyName);
              MapService.updateValue(_initValue!, propertyName, val);
            }
          }
        }
        // _initValue!['name'] = v['name'];
        // _initValue!['title'] = v['title'];
      } else {
        _initValue = v;
      }
      // widget.data[widget.property.dbName] = _initValue;
      if (widget.onChanged != null &&
          (_initValue != null || valueBefore != null)) {
        widget.onChanged?.call(_initValue);
      }
      MapService.updateValue(widget.data, widget.property.dbName, _initValue);
      setState(() {});
      context != null ? Navigator.pop(context) : print("null Navigator");
    } catch (err, trac) {
      print('xxxx error select from list $err');
      print(trac);
    }
  }

  _onSaveNewItem(BuildContext context, newDataRaw) async {
    try {
      var validateResult =
          EntityService.validateEntity(_entity, newDataRaw, widget.isAddNew);
      if (!validateResult.success) {
        AppNavigator.showMessage(
            context, validateResult.message ?? 'Error', MessageType.error);
        return;
      }
      var addResult =
          await EntityService.saveEntity(_entity, newDataRaw, context, true);
      if (addResult.success) {
        _initValue = addResult.data;
        widget.data[widget.property.dbName] = _initValue;
        widget.onChanged?.call(_initValue!);
        Navigator.pop(context, true); // close the add new item's form
      } else {
        AppNavigator.showMessage(
            context, addResult.message ?? 'Error', MessageType.error);
      }
    } catch (err, trace) {
      print('XXXXXXXXX erorr adding item');
      print(err);
      print(trace);
      AppNavigator.showMessage(context, 'Error Adding Item', MessageType.error);
    }
  }

  _openSearchDialog() async {
    widget.onOpen?.call();
    await AppNavigator.showAppDialog(
        context,
        (listDialogContext) => Column(
              mainAxisAlignment: MainAxisAlignment.center,
              crossAxisAlignment: CrossAxisAlignment.center,
              children: <Widget>[
                Expanded(child: Container()),
                Container(
                    width: min(500, MediaQuery.of(context).size.width),
                    height: min(
                        800,
                        MediaQuery.of(context).size.height -
                            MediaQuery.of(context).padding.top -
                            MediaQuery.of(context).padding.bottom -
                            50),
                    child: Card(
                        child: Column(
                            mainAxisAlignment: MainAxisAlignment.start,
                            crossAxisAlignment: CrossAxisAlignment.stretch,
                            children: <Widget>[
                          Container(
                              color: ThemeService.backgroundSecondColor,
                              child: Row(
                                  mainAxisAlignment:
                                      MainAxisAlignment.spaceBetween,
                                  children: <Widget>[
                                    Padding(
                                      padding: EdgeInsets.only(left: 10),
                                      child: getLableWidget(),
                                    ),
                                    IconButton(
                                        icon: Icon(Icons.close),
                                        onPressed: () {
                                          widget.onClose?.call();
                                          Navigator.pop(listDialogContext);
                                        })
                                  ])),
                          Expanded(
                              child: EntityListSearchComponent(
                            property: widget.property,
                            entity: widget.entity,
                            url: widget.url ?? '',
                            data: widget.data,
                            // isFilter: widget.isFilter ,
                            // filterValue: widget.filterValue,
                            // filterField: widget.filterField,
                            urlFilterData: widget.data,
                            onClick: (v) =>
                                _onItemSelectedFromList(listDialogContext, v),
                          )),
                          TextButton(
                            child: TecfyText(trans('Add New Item'),
                                style: ThemeService.flatButtonTextStyle
                                    .copyWith(
                                        decoration: TextDecoration.underline)),
                            onPressed: () async {
                              Map<String, dynamic> newDataRaw =
                                  new Map<String, dynamic>();
                              var addNewResult =
                                  await AppNavigator.showAppDialogEx(
                                      listDialogContext,
                                      () => _getAddNewDialog(
                                          listDialogContext, newDataRaw),
                                      size: DialogBoxSizes.large);
                              if (addNewResult ?? false)
                                Navigator.pop(
                                    listDialogContext); // close the search form
                            },
                          )
                        ]))),
                Expanded(child: Container()),
              ],
            ));
    //loadData();
  }

  Widget _getAddNewDialog(addDialogContext, newDataRaw) {
    return Container(
      constraints: BoxConstraints(maxHeight: 500, maxWidth: 500),
      child: Center(
        child: Card(
          child: ListView(
            children: <Widget>[
              InLineForm(widget.entityId, true,
                  entity: _entity!,
                  inPopup: true,
                  data: newDataRaw,
                  onChanged: (v) {}),
              Padding(
                  padding: EdgeInsets.symmetric(horizontal: 20),
                  child: Row(
                    children: <Widget>[
                      Expanded(child: Container()),
                      Expanded(
                          child: AppButtonPost(
                              label: trans('Save'),
                              onPressed: () => _onSaveNewItem(
                                  addDialogContext, newDataRaw))),
                      SizedBox(width: 10),
                      Expanded(
                          child: Container(
                              height: 50.0,
                              child: TextButton(
                                  child: TecfyText(trans('Cancel'),
                                      style: ThemeService.flatButtonTextStyle),
                                  onPressed: () =>
                                      Navigator.pop(addDialogContext, false)))),
                      Expanded(child: Container()),
                    ],
                  )),
            ],
          ),
        ),
      ),
    );
  }

  @override
  Widget build(BuildContext context) {
    if (_initValue != widget.data[widget.property.dbName] || _initValue == null)
      _init();
    // print('----------- ${_entity!.displayField.dbName}');
    // if (_dataValue != null && _entity?.displayField.dbName != null)
    //   print("${_dataValue![_entity!.displayField.dbName]}");
    return _initValue != null
        ? Stack(children: [
            MaterialButton(
                padding: EdgeInsets.all(0),
                onPressed: widget.readOnly ? null : _openSearchDialog,
                child: widget.label == null
                    ? TecfyText(
                        _initValue == null
                            ? trans('Click To Select')
                            : (_initValue![_entity!.displayField.dbName] is Map
                                ? _initValue![_entity!.displayField.dbName][
                                        LocalConfigService.languagePrefix
                                            .toLowerCase()] ??
                                    'N/A'
                                : _initValue![_entity!.displayField.dbName]),
                        style: TextStyle(color: Colors.black, fontSize: 15))
                    : Column(
                        crossAxisAlignment: CrossAxisAlignment.start,
                        children: <Widget>[
                          SizedBox(height: 10),
                          getLableWidget(),
                          TecfyText(
                              _initValue == null
                                  ? trans('Click To Select')
                                  : (_initValue![_entity!.displayField.dbName]
                                          is Map
                                      ? _initValue![
                                                  _entity!.displayField.dbName][
                                              LocalConfigService.languagePrefix
                                                  .toLowerCase()]
                                          .toString()
                                      : _initValue![
                                              _entity!.displayField.dbName] ??
                                          'No Name'),
                              style:
                                  TextStyle(color: Colors.black, fontSize: 15)),
                          SizedBox(height: 10),
                          Container(
                              height: 1, color: ThemeService.textSubColor),
                          SizedBox(height: 5),
                        ],
                      )),
            widget.readOnly || widget.isRequired
                ? SizedBox()
                : Align(
                    alignment: Alignment.topLeft,
                    child: IconButton(
                        onPressed: () {
                          _onItemSelectedFromList(null, null);
                        },
                        icon: Icon(Icons.close, color: Colors.grey),
                        tooltip: trans("Clean")),
                  )
          ])
        : MaterialButton(
            padding: EdgeInsets.all(0),
            onPressed: _openSearchDialog,
            child: widget.label == null
                ? TecfyText(
                    _initValue == null
                        ? trans('Click To Select')
                        : (_initValue![_entity!.displayField.dbName] ?? 'N/A'),
                    style: TextStyle(color: Colors.black, fontSize: 15))
                : Column(
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children: <Widget>[
                      SizedBox(height: 10),
                      getLableWidget(),
                      TecfyText(
                          _initValue == null
                              ? trans('Click To Select')
                              : (_initValue![_entity!.displayField.dbName]
                                      ?.toString() ??
                                  'N/A'),
                          style: TextStyle(color: Colors.black, fontSize: 15)),
                      SizedBox(height: 10),
                      Container(height: 1, color: ThemeService.textSubColor),
                      SizedBox(height: 5),
                    ],
                  ));
  }

  Widget getLableWidget() {
    return Row(
      children: [
        Tooltip(
            message: widget.description ?? '',
            child: TecfyText(widget.label ?? '',
                style:
                    TextStyle(color: ThemeService.textSubColor, fontSize: 11))),
        widget.isRequired
            ? TecfyText('*', style: TextStyle(color: Colors.red, fontSize: 11))
            : SizedBox(),
      ],
    );
  }
}
